@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2005-2007
+   Copyright (C) Jay Sorg 2005-2008
 */
 
 /**
@@ -22,7 +22,7 @@
  * @file access.h
  * @brief User access control definitions
  * @author Simone Fedele
- * 
+ *
  */
 
 #ifndef ACCESS_H
@@ -38,4 +38,14 @@
 int DEFAULT_CC
 access_login_allowed(char* user);
 
+/**
+ *
+ * @brief Checks if the user is allowed to access the terminal server for management
+ * @param user the user to check
+ * @return 0 if access is denied, !=0 if allowed
+ *
+ */
+int DEFAULT_CC
+access_login_mng_allowed(char* user);
+
 #endif
