@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2005-2007
+   Copyright (C) Jay Sorg 2005-2008
 */
 
 /**
@@ -31,51 +31,65 @@
 
 extern int g_sck;
 extern int g_pid;
-extern struct config_sesman g_cfg;
+extern struct config_sesman* g_cfg; /* in sesman.c */
+extern tbus g_term_event;
 
 /******************************************************************************/
 void DEFAULT_CC
 sig_sesman_shutdown(int sig)
 {
-  log_message(LOG_LEVEL_INFO, "shutting down sesman %d", 1);
+  char pid_file[256];
+
+  log_message(&(g_cfg->log), LOG_LEVEL_INFO, "shutting down sesman %d", 1);
 
   if (g_getpid() != g_pid)
   {
-    LOG_DBG("g_getpid() [%d] differs from g_pid [%d]", (g_getpid()), g_pid);
+    LOG_DBG(&(g_cfg->log), "g_getpid() [%d] differs from g_pid [%d]", (g_getpid()), g_pid);
     return;
   }
 
-  LOG_DBG(" - getting signal %d pid %d", sig, g_getpid());
+  LOG_DBG(&(g_cfg->log), " - getting signal %d pid %d", sig, g_getpid());
+
+  g_set_wait_obj(g_term_event);
 
   g_tcp_close(g_sck);
 
   session_sigkill_all();
 
-  g_file_delete(SESMAN_PID_FILE);
+  g_snprintf(pid_file, 255, "%s/xrdp-sesman.pid", XRDP_PID_PATH);
+  g_file_delete(pid_file);
 }
 
 /******************************************************************************/
 void DEFAULT_CC
 sig_sesman_reload_cfg(int sig)
 {
-  struct config_sesman cfg;
+  struct config_sesman *cfg;
 
-  log_message(LOG_LEVEL_WARNING, "receiving SIGHUP %d", 1);
+  log_message(&(g_cfg->log), LOG_LEVEL_WARNING, "receiving SIGHUP %d", 1);
 
   if (g_getpid() != g_pid)
   {
-    LOG_DBG("g_getpid() [%d] differs from g_pid [%d]", g_getpid(), g_pid);
+    LOG_DBG(&(g_cfg->log), "g_getpid() [%d] differs from g_pid [%d]", g_getpid(), g_pid);
     return;
   }
 
-  if (config_read(&cfg) != 0)
+  cfg = g_malloc(sizeof(struct config_sesman), 1);
+  if (0 == cfg)
   {
-    log_message(LOG_LEVEL_ERROR, "error reading config - keeping old cfg");
+    log_message(&(g_cfg->log), LOG_LEVEL_ERROR, "error creating new config:  - keeping old cfg");
     return;
   }
+
+  if (config_read(cfg) != 0)
+  {
+    log_message(&(g_cfg->log), LOG_LEVEL_ERROR, "error reading config - keeping old cfg");
+    return;
+  }
+#warning FIXME reload configuration must NOT damage logging!
   g_cfg = cfg;
 
-  log_message(LOG_LEVEL_INFO, "configuration reloaded");
+  log_message(&(g_cfg->log), LOG_LEVEL_INFO, "configuration reloaded");
 }
 
 /******************************************************************************/
@@ -115,35 +129,43 @@ sig_handler_thread(void* arg)
   sigaddset(&waitmask, SIGHUP);
   sigaddset(&waitmask, SIGCHLD);
   sigaddset(&waitmask, SIGTERM);
+  sigaddset(&waitmask, SIGKILL);
+  sigaddset(&waitmask, SIGINT);
+
 //  sigaddset(&waitmask, SIGFPE);
 //  sigaddset(&waitmask, SIGILL);
 //  sigaddset(&waitmask, SIGSEGV);
 
   do
   {
-    LOG_DBG("calling sigwait()",0);
+    LOG_DBG(&(g_cfg->log), "calling sigwait()",0);
     sigwait(&waitmask, &recv_signal);
-
     switch (recv_signal)
     {
       case SIGHUP:
         //reload cfg
-        LOG_DBG("sesman received SIGHUP",0);
+	//we must stop & restart logging, or copy logging cfg!!!!
+        LOG_DBG(&(g_cfg->log), "sesman received SIGHUP", 0);
         //return 0;
         break;
       case SIGCHLD:
         /* a session died */
-        LOG_DBG("sesman received SIGCHLD",0);
+        LOG_DBG(&(g_cfg->log), "sesman received SIGCHLD", 0);
         sig_sesman_session_end(SIGCHLD);
         break;
-      /*case SIGKILL;
-        / * we die * /
-        LOG_DBG("sesman received SIGKILL",0);
+      case SIGINT:
+        /* we die */
+        LOG_DBG(&(g_cfg->log), "sesman received SIGINT", 0);
         sig_sesman_shutdown(recv_signal);
-        break;*/
+        break;
+      case SIGKILL:
+        /* we die */
+        LOG_DBG(&(g_cfg->log), "sesman received SIGKILL", 0);
+        sig_sesman_shutdown(recv_signal);
+        break;
       case SIGTERM:
         /* we die */
-        LOG_DBG("sesman received SIGTERM",0);
+        LOG_DBG(&(g_cfg->log), "sesman received SIGTERM", 0);
         sig_sesman_shutdown(recv_signal);
         break;
     }
@@ -151,4 +173,3 @@ sig_handler_thread(void* arg)
 
   return 0;
 }
-
