@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2007
+   Copyright (C) Jay Sorg 2004-2009
 
    module manager
 
@@ -40,7 +40,7 @@ xrdp_mm_create(struct xrdp_wm* owner)
 /*****************************************************************************/
 /* called from main thread */
 static long DEFAULT_CC
-sync_unload(long param1, long param2)
+xrdp_mm_sync_unload(long param1, long param2)
 {
   return g_free_library(param1);
 }
@@ -48,9 +48,14 @@ sync_unload(long param1, long param2)
 /*****************************************************************************/
 /* called from main thread */
 static long DEFAULT_CC
-sync_load(long param1, long param2)
+xrdp_mm_sync_load(long param1, long param2)
 {
-  return g_load_library((char*)param1);
+  long rv;
+  char* libname;
+
+  libname = (char*)param1;
+  rv = g_load_library(libname);
+  return rv;
 }
 
 /*****************************************************************************/
@@ -61,13 +66,18 @@ xrdp_mm_module_cleanup(struct xrdp_mm* self)
   {
     if (self->mod_exit != 0)
     {
+      /* let the module cleanup */
       self->mod_exit(self->mod);
     }
   }
   if (self->mod_handle != 0)
   {
-    g_xrdp_sync(sync_unload, self->mod_handle, 0);
+    /* main thread unload */
+    g_xrdp_sync(xrdp_mm_sync_unload, self->mod_handle, 0);
   }
+  trans_delete(self->chan_trans);
+  self->chan_trans = 0;
+  self->chan_trans_up = 0;
   self->mod_init = 0;
   self->mod_exit = 0;
   self->mod = 0;
@@ -82,104 +92,16 @@ xrdp_mm_delete(struct xrdp_mm* self)
   {
     return;
   }
-  /* free any modual stuff */
+  /* free any module stuff */
   xrdp_mm_module_cleanup(self);
-  if (self->sck != 0)
-  {
-    g_tcp_close(self->sck);
-    self->sck = 0;
-  }
+  trans_delete(self->sesman_trans);
+  self->sesman_trans = 0;
+  self->sesman_trans_up = 0;
   list_delete(self->login_names);
   list_delete(self->login_values);
   g_free(self);
 }
 
-/******************************************************************************/
-/* returns error */
-static int APP_CC
-xrdp_mm_recv(struct xrdp_mm* self, char* data, int len)
-{
-  int rcvd;
-
-  if (self->sck_closed)
-  {
-    return 1;
-  }
-  while (len > 0)
-  {
-    rcvd = g_tcp_recv(self->sck, data, len, 0);
-    if (rcvd == -1)
-    {
-      if (g_tcp_last_error_would_block(self->sck))
-      {
-        if (g_is_term())
-        {
-          return 1;
-        }
-        g_tcp_can_recv(self->sck, 10);
-      }
-      else
-      {
-        return 1;
-      }
-    }
-    else if (rcvd == 0)
-    {
-      self->sck_closed = 1;
-      return 1;
-    }
-    else
-    {
-      data += rcvd;
-      len -= rcvd;
-    }
-  }
-  return 0;
-}
-
-/*****************************************************************************/
-/* returns error */
-static int APP_CC
-xrdp_mm_send(struct xrdp_mm* self, char* data, int len)
-{
-  int sent;
-
-  if (self->sck_closed)
-  {
-    return 1;
-  }
-  while (len > 0)
-  {
-    sent = g_tcp_send(self->sck, data, len, 0);
-    if (sent == -1)
-    {
-      if (g_tcp_last_error_would_block(self->sck))
-      {
-        if (g_is_term())
-        {
-          return 1;
-        }
-        g_tcp_can_send(self->sck, 10);
-      }
-      else
-      {
-        return 1;
-      }
-    }
-    else if (sent == 0)
-    {
-      self->sck_closed = 1;
-      return 1;
-    }
-    else
-    {
-      data += sent;
-      len -= sent;
-    }
-  }
-  return 0;
-}
-
 /*****************************************************************************/
 static int APP_CC
 xrdp_mm_send_login(struct xrdp_mm* self)
@@ -224,8 +146,7 @@ xrdp_mm_send_login(struct xrdp_mm* self)
     xrdp_wm_log_msg(self->wm, "error finding username and password");
     return 1;
   }
-  make_stream(s);
-  init_stream(s, 8192);
+  s = trans_get_out_s(self->sesman_trans, 8192);
   s_push_layer(s, channel_hdr, 8);
   /* this code is either 0 for Xvnc or 10 for X11rdp */
   out_uint16_be(s, self->code);
@@ -238,23 +159,38 @@ xrdp_mm_send_login(struct xrdp_mm* self)
   out_uint16_be(s, self->wm->screen->width);
   out_uint16_be(s, self->wm->screen->height);
   out_uint16_be(s, self->wm->screen->bpp);
+  /* send domain */
+  index = g_strlen(self->wm->client_info->domain);
+  out_uint16_be(s, index);
+  out_uint8a(s, self->wm->client_info->domain, index);
+  /* send program / shell */
+  index = g_strlen(self->wm->client_info->program);
+  out_uint16_be(s, index);
+  out_uint8a(s, self->wm->client_info->program, index);
+  /* send directory */
+  index = g_strlen(self->wm->client_info->directory);
+  out_uint16_be(s, index);
+  out_uint8a(s, self->wm->client_info->directory, index);
   s_mark_end(s);
   s_pop_layer(s, channel_hdr);
   out_uint32_be(s, 0); /* version */
-  index = s->end - s->data;
+  index = (int)(s->end - s->data);
   out_uint32_be(s, index); /* size */
-  rv = xrdp_mm_send(self, s->data, index);
-  free_stream(s);
+  rv = trans_force_write(self->sesman_trans);
+  if (rv != 0)
+  {
+    xrdp_wm_log_msg(self->wm, "xrdp_mm_send_login: xrdp_mm_send failed");
+  }
   return rv;
 }
 
 /*****************************************************************************/
 /* returns error */
-/* this goes through the login_names looking for one called 'lib'
+/* this goes through the login_names looking for one called 'aname'
    then it copies the corisponding login_values item into 'dest'
    'dest' must be at least 'dest_len' + 1 bytes in size */
 static int APP_CC
-xrdp_mm_get_lib(struct xrdp_mm* self, char* dest, int dest_len)
+xrdp_mm_get_value(struct xrdp_mm* self, char* aname, char* dest, int dest_len)
 {
   char* name;
   char* value;
@@ -274,7 +210,7 @@ xrdp_mm_get_lib(struct xrdp_mm* self, char* dest, int dest_len)
     {
       break;
     }
-    if (g_strcasecmp(name, "lib") == 0)
+    if (g_strcasecmp(name, aname) == 0)
     {
       g_strncpy(dest, value, dest_len);
       rv = 0;
@@ -289,25 +225,30 @@ xrdp_mm_setup_mod1(struct xrdp_mm* self)
 {
   void* func;
   char lib[256];
+  char text[256];
 
   if (self == 0)
   {
     return 1;
   }
   lib[0] = 0;
-  if (xrdp_mm_get_lib(self, lib, 255) != 0)
+  if (xrdp_mm_get_value(self, "lib", lib, 255) != 0)
   {
-    g_writeln("error finding lib");
+    g_snprintf(text, 255, "no library name specified in xrdp.ini, please add "
+               "lib=libxrdp-vnc.so or similar");
+    xrdp_wm_log_msg(self->wm, text);
     return 1;
   }
   if (lib[0] == 0)
   {
-    g_writeln("error finding lib");
+    g_snprintf(text, 255, "empty library name specified in xrdp.ini, please "
+               "add lib=libxrdp-vnc.so or similar");
+    xrdp_wm_log_msg(self->wm, text);
     return 1;
   }
   if (self->mod_handle == 0)
   {
-    self->mod_handle = g_xrdp_sync(sync_load, (long)lib, 0);
+    self->mod_handle = g_xrdp_sync(xrdp_mm_sync_load, (long)lib, 0);
     if (self->mod_handle != 0)
     {
       func = g_get_proc_address(self->mod_handle, "mod_init");
@@ -317,7 +258,9 @@ xrdp_mm_setup_mod1(struct xrdp_mm* self)
       }
       if (func == 0)
       {
-        g_writeln("error finding proc mod_init in %s", lib);
+        g_snprintf(text, 255, "error finding proc mod_init in %s, not a valid "
+                   "xrdp backend", lib);
+        xrdp_wm_log_msg(self->wm, text);
       }
       self->mod_init = (struct xrdp_mod* (*)(void))func;
       func = g_get_proc_address(self->mod_handle, "mod_exit");
@@ -327,17 +270,26 @@ xrdp_mm_setup_mod1(struct xrdp_mm* self)
       }
       if (func == 0)
       {
-        g_writeln("error finding proc mod_exit in %s", lib);
+        g_snprintf(text, 255, "error finding proc mod_exit in %s, not a valid "
+                   "xrdp backend", lib);
+        xrdp_wm_log_msg(self->wm, text);
       }
       self->mod_exit = (int (*)(struct xrdp_mod*))func;
       if ((self->mod_init != 0) && (self->mod_exit != 0))
       {
         self->mod = self->mod_init();
+        if (self->mod != 0)
+        {
+          g_writeln("loaded modual '%s' ok, interface size %d, version %d", lib,
+                    self->mod->size, self->mod->version);
+        }
       }
     }
     else
     {
-      g_writeln("error loading %s", lib);
+      g_snprintf(text, 255, "error loading %s specified in xrdp.ini, please "
+                 "add a valid entry like lib=libxrdp-vnc.so or similar", lib);
+      xrdp_wm_log_msg(self->wm, text);
     }
     if (self->mod != 0)
     {
@@ -386,19 +338,21 @@ xrdp_mm_setup_mod2(struct xrdp_mm* self)
   char* value;
   int i;
   int rv;
+  int key_flags;
+  int device_flags;
 
   rv = 1;
   text[0] = 0;
-  if (!(self->wm->pro_layer->term))
+  if (!g_is_wait_obj_set(self->wm->pro_layer->self_term_event))
   {
     if (self->mod->mod_start(self->mod, self->wm->screen->width,
                              self->wm->screen->height,
                              self->wm->screen->bpp) != 0)
     {
-      self->wm->pro_layer->term = 1; /* kill session */
+      g_set_wait_obj(self->wm->pro_layer->self_term_event); /* kill session */
     }
   }
-  if (!(self->wm->pro_layer->term))
+  if (!g_is_wait_obj_set(self->wm->pro_layer->self_term_event))
   {
     if (self->display > 0)
     {
@@ -412,11 +366,11 @@ xrdp_mm_setup_mod2(struct xrdp_mm* self)
       }
       else
       {
-        self->wm->pro_layer->term = 1; /* kill session */
+        g_set_wait_obj(self->wm->pro_layer->self_term_event); /* kill session */
       }
     }
   }
-  if (!(self->wm->pro_layer->term))
+  if (!g_is_wait_obj_set(self->wm->pro_layer->self_term_event))
   {
     /* this adds the port to the end of the list, it will already be in
        the list as -1
@@ -443,9 +397,237 @@ xrdp_mm_setup_mod2(struct xrdp_mm* self)
       rv = 0;
     }
   }
+  if (rv == 0)
+  {
+    /* sync modifiers */
+    key_flags = 0;
+    device_flags = 0;
+    if (self->wm->scroll_lock)
+    {
+      key_flags |= 1;
+    }
+    if (self->wm->num_lock)
+    {
+      key_flags |= 2;
+    }
+    if (self->wm->caps_lock)
+    {
+      key_flags |= 4;
+    }
+    if (self->mod != 0)
+    {
+      if (self->mod->mod_event != 0)
+      {
+        self->mod->mod_event(self->mod, 17, key_flags, device_flags,
+                             key_flags, device_flags);
+      }
+    }
+  }
   return rv;
 }
 
+/*****************************************************************************/
+/* returns error
+   send a list of channels to the channel handler */
+static int APP_CC
+xrdp_mm_trans_send_channel_setup(struct xrdp_mm* self, struct trans* trans)
+{
+  int index;
+  int chan_id;
+  int chan_flags;
+  int size;
+  struct stream* s;
+  char chan_name[256];
+
+  s = trans_get_out_s(trans, 8192);
+  if (s == 0)
+  {
+    return 1;
+  }
+  s_push_layer(s, iso_hdr, 8);
+  s_push_layer(s, mcs_hdr, 8);
+  s_push_layer(s, sec_hdr, 2);
+  index = 0;
+  while (libxrdp_query_channel(self->wm->session, index, chan_name,
+                               &chan_flags) == 0)
+  {
+    chan_id = libxrdp_get_channel_id(self->wm->session, chan_name);
+    out_uint8a(s, chan_name, 8);
+    out_uint16_le(s, chan_id);
+    out_uint16_le(s, chan_flags);
+    index++;
+  }
+  s_mark_end(s);
+  s_pop_layer(s, sec_hdr);
+  out_uint16_le(s, index);
+  s_pop_layer(s, mcs_hdr);
+  size = (int)(s->end - s->p);
+  out_uint32_le(s, 3); /* msg id */
+  out_uint32_le(s, size); /* msg size */
+  s_pop_layer(s, iso_hdr);
+  size = (int)(s->end - s->p);
+  out_uint32_le(s, 0); /* version */
+  out_uint32_le(s, size); /* block size */
+  return trans_force_write(trans);
+}
+
+/*****************************************************************************/
+/* returns error */
+static int APP_CC
+xrdp_mm_trans_send_channel_data_response(struct xrdp_mm* self,
+                                         struct trans* trans)
+{
+  struct stream* s;
+
+  s = trans_get_out_s(trans, 8192);
+  if (s == 0)
+  {
+    return 1;
+  }
+  out_uint32_le(s, 0); /* version */
+  out_uint32_le(s, 8 + 8); /* size */
+  out_uint32_le(s, 7); /* msg id */
+  out_uint32_le(s, 8); /* size */
+  s_mark_end(s);
+  return trans_force_write(trans);
+}
+
+/*****************************************************************************/
+/* returns error
+   init is done, sent channel setup */
+static int APP_CC
+xrdp_mm_trans_process_init_response(struct xrdp_mm* self, struct trans* trans)
+{
+  return xrdp_mm_trans_send_channel_setup(self, trans);
+}
+
+/*****************************************************************************/
+/* returns error
+   data coming in from the channel handler, send it to the client */
+static int APP_CC
+xrdp_mm_trans_process_channel_data(struct xrdp_mm* self, struct trans* trans)
+{
+  struct stream* s;
+  int size;
+  int total_size;
+  int chan_id;
+  int chan_flags;
+  int rv;
+
+  s = trans_get_in_s(trans);
+  if (s == 0)
+  {
+    return 1;
+  }
+  in_uint16_le(s, chan_id);
+  in_uint16_le(s, chan_flags);
+  in_uint16_le(s, size);
+  in_uint32_le(s, total_size);
+  rv = xrdp_mm_trans_send_channel_data_response(self, trans);
+  if (rv == 0)
+  {
+    rv = libxrdp_send_to_channel(self->wm->session, chan_id, s->p, size, total_size,
+                                 chan_flags);
+  }
+  return rv;
+}
+
+/*****************************************************************************/
+/* returns error
+   process a message for the channel handler */
+static int APP_CC
+xrdp_mm_chan_process_msg(struct xrdp_mm* self, struct trans* trans,
+                         struct stream* s)
+{
+  int rv;
+  int id;
+  int size;
+  char* next_msg;
+
+  rv = 0;
+  while (s_check_rem(s, 8))
+  {
+    next_msg = s->p;
+    in_uint32_le(s, id);
+    in_uint32_le(s, size);
+    next_msg += size;
+    switch (id)
+    {
+      case 2: /* channel init response */
+        rv = xrdp_mm_trans_process_init_response(self, trans);
+        break;
+      case 4: /* channel setup response */
+        break;
+      case 6: /* channel data response */
+        break;
+      case 8: /* channel data */
+        rv = xrdp_mm_trans_process_channel_data(self, trans);
+        break;
+      default:
+        g_writeln("xrdp_mm_chan_process_msg: unknown id %d", id);
+        break;
+    }
+    if (rv != 0)
+    {
+      break;
+    }
+    s->p = next_msg;
+  }
+  return rv;
+}
+
+/*****************************************************************************/
+/* this is callback from trans obj
+   returns error */
+static int APP_CC
+xrdp_mm_chan_data_in(struct trans* trans)
+{
+  struct xrdp_mm* self;
+  struct stream* s;
+  int id;
+  int size;
+  int error;
+
+  if (trans == 0)
+  {
+    return 1;
+  }
+  self = (struct xrdp_mm*)(trans->callback_data);
+  s = trans_get_in_s(trans);
+  if (s == 0)
+  {
+    return 1;
+  }
+  in_uint32_le(s, id);
+  in_uint32_le(s, size);
+  error = trans_force_read(trans, size - 8);
+  if (error == 0)
+  {
+    /* here, the entire message block is read in, process it */
+    error = xrdp_mm_chan_process_msg(self, trans, s);
+  }
+  return error;
+}
+
+/*****************************************************************************/
+static int APP_CC
+xrdp_mm_chan_send_init(struct xrdp_mm* self)
+{
+  struct stream* s;
+
+  s = trans_get_out_s(self->chan_trans, 8192);
+  if (s == 0)
+  {
+    return 1;
+  }
+  out_uint32_le(s, 0); /* version */
+  out_uint32_le(s, 8 + 8); /* size */
+  out_uint32_le(s, 1); /* msg id */
+  out_uint32_le(s, 8); /* size */
+  s_mark_end(s);
+  return trans_force_write(self->chan_trans);
+}
+
 /*****************************************************************************/
 static int APP_CC
 xrdp_mm_process_login_response(struct xrdp_mm* self, struct stream* s)
@@ -453,7 +635,10 @@ xrdp_mm_process_login_response(struct xrdp_mm* self, struct stream* s)
   int ok;
   int display;
   int rv;
+  int index;
   char text[256];
+  char ip[256];
+  char port[256];
 
   rv = 0;
   in_uint16_be(s, ok);
@@ -461,34 +646,211 @@ xrdp_mm_process_login_response(struct xrdp_mm* self, struct stream* s)
   if (ok)
   {
     self->display = display;
-    g_snprintf(text, 255, "login successful for display %d", display);
+    g_snprintf(text, 255, "xrdp_mm_process_login_response: login successful "
+               "for display %d", display);
     xrdp_wm_log_msg(self->wm, text);
     if (xrdp_mm_setup_mod1(self) == 0)
     {
       if (xrdp_mm_setup_mod2(self) == 0)
       {
-        self->wm->login_mode = 10;
-        self->wm->pro_layer->app_sck = self->mod->sck;
+        xrdp_mm_get_value(self, "ip", ip, 255);
+        xrdp_wm_set_login_mode(self->wm, 10);
+        self->wm->dragging = 0;
+        /* connect channel redir */
+        if (strcmp(ip, "127.0.0.1") == 0)
+        {
+          /* unix socket */
+          self->chan_trans = trans_create(2, 8192, 8192);
+          g_snprintf(port, 255, "/tmp/xrdp_chansrv_socket_%d", 7200 + display);
+        }
+        else
+        {
+          /* tcp */
+          self->chan_trans = trans_create(1, 8192, 8192);
+          g_snprintf(port, 255, "%d", 7200 + display);
+        }
+        self->chan_trans->trans_data_in = xrdp_mm_chan_data_in;
+        self->chan_trans->header_size = 8;
+        self->chan_trans->callback_data = self;
+        /* try to connect up to 4 times */
+        for (index = 0; index < 4; index++)
+        {
+          if (trans_connect(self->chan_trans, ip, port, 3000) == 0)
+          {
+            self->chan_trans_up = 1;
+            break;
+          }
+          g_sleep(1000);
+          g_writeln("xrdp_mm_process_login_response: connect failed "
+                    "trying again...");
+        }
+        if (!(self->chan_trans_up))
+        {
+          g_writeln("xrdp_mm_process_login_response: error in trans_connect "
+                    "chan");
+        }
+        if (self->chan_trans_up)
+        {
+          if (xrdp_mm_chan_send_init(self) != 0)
+          {
+            g_writeln("xrdp_mm_process_login_response: error in "
+                      "xrdp_mm_chan_send_init");
+          }
+        }
       }
     }
   }
   else
   {
-    xrdp_wm_log_msg(self->wm, "login failed");
+    xrdp_wm_log_msg(self->wm, "xrdp_mm_process_login_response: login failed");
   }
-  /* close socket */
-  g_tcp_close(self->sck);
-  self->sck = 0;
+  self->delete_sesman_trans = 1;
   self->connected_state = 0;
   if (self->wm->login_mode != 10)
   {
-    self->wm->pro_layer->app_sck = 0;
-    self->wm->login_mode = 11;
+    xrdp_wm_set_login_mode(self->wm, 11);
     xrdp_mm_module_cleanup(self);
   }
   return rv;
 }
 
+/*****************************************************************************/
+static int
+xrdp_mm_get_sesman_port(char* port, int port_bytes)
+{
+  int fd;
+  int error;
+  int index;
+  char* val;
+  char cfg_file[256];
+  struct list* names;
+  struct list* values;
+
+  /* default to port 3350 */
+  g_strncpy(port, "3350", port_bytes - 1);
+  /* see if port is in xrdp.ini file */
+  g_snprintf(cfg_file, 255, "%s/sesman.ini", XRDP_CFG_PATH);
+  fd = g_file_open(cfg_file);
+  if (fd > 0)
+  {
+    names = list_create();
+    names->auto_free = 1;
+    values = list_create();
+    values->auto_free = 1;
+    if (file_read_section(fd, "Globals", names, values) == 0)
+    {
+      for (index = 0; index < names->count; index++)
+      {
+        val = (char*)list_get_item(names, index);
+        if (val != 0)
+        {
+          if (g_strcasecmp(val, "ListenPort") == 0)
+          {
+            val = (char*)list_get_item(values, index);
+            error = g_atoi(val);
+            if ((error > 0) && (error < 65000))
+            {
+              g_strncpy(port, val, port_bytes - 1);
+            }
+            break;
+          }
+        }
+      }
+    }
+    list_delete(names);
+    list_delete(values);
+    g_file_close(fd);
+  }
+  return 0;
+}
+
+/*****************************************************************************/
+/* returns error
+   data coming from client that need to go to channel handler */
+int APP_CC
+xrdp_mm_process_channel_data(struct xrdp_mm* self, tbus param1, tbus param2,
+                             tbus param3, tbus param4)
+{
+  struct stream* s;
+  int rv;
+  int length;
+  int total_length;
+  int flags;
+  int id;
+  char* data;
+
+  rv = 0;
+  if ((self->chan_trans != 0) && self->chan_trans_up)
+  {
+    s = trans_get_out_s(self->chan_trans, 8192);
+    if (s != 0)
+    {
+      id = LOWORD(param1);
+      flags = HIWORD(param1);
+      length = param2;
+      data = (char*)param3;
+      total_length = param4;
+      if (total_length < length)
+      {
+        g_writeln("warning in xrdp_mm_process_channel_data total_len < length");
+        total_length = length;
+      }
+      out_uint32_le(s, 0); /* version */
+      out_uint32_le(s, 8 + 8 + 2 + 2 + 2 + 4 + length);
+      out_uint32_le(s, 5); /* msg id */
+      out_uint32_le(s, 8 + 2 + 2 + 2 + 4 + length);
+      out_uint16_le(s, id);
+      out_uint16_le(s, flags);
+      out_uint16_le(s, length);
+      out_uint32_le(s, total_length);
+      out_uint8a(s, data, length);
+      s_mark_end(s);
+      rv = trans_force_write(self->chan_trans);
+    }
+  }
+  return rv;
+}
+
+/*****************************************************************************/
+static int APP_CC
+xrdp_mm_sesman_data_in(struct trans* trans)
+{
+  struct xrdp_mm* self;
+  struct stream* s;
+  int version;
+  int size;
+  int error;
+  int code;
+
+  if (trans == 0)
+  {
+    return 1;
+  }
+  self = (struct xrdp_mm*)(trans->callback_data);
+  s = trans_get_in_s(trans);
+  if (s == 0)
+  {
+    return 1;
+  }
+  in_uint32_be(s, version);
+  in_uint32_be(s, size);
+  error = trans_force_read(trans, size - 8);
+  if (error == 0)
+  {
+    in_uint16_be(s, code);
+    switch (code)
+    {
+      case 3:
+        error = xrdp_mm_process_login_response(self, s);
+        break;
+      default:
+        g_writeln("xrdp_mm_sesman_data_in: unknown code %d", code);
+        break;
+    }
+  }
+  return error;
+}
+
 /*****************************************************************************/
 int APP_CC
 xrdp_mm_connect(struct xrdp_mm* self)
@@ -506,6 +868,7 @@ xrdp_mm_connect(struct xrdp_mm* self)
   char ip[256];
   char errstr[256];
   char text[256];
+  char port[8];
 
   rv = 0;
   use_sesman = 0;
@@ -532,52 +895,41 @@ xrdp_mm_connect(struct xrdp_mm* self)
   {
     ok = 0;
     errstr[0] = 0;
-    self->sck = g_tcp_socket();
-    g_tcp_set_non_blocking(self->sck);
-    g_snprintf(text, 255, "connecting to sesman ip %s port 3350", ip);
+    trans_delete(self->sesman_trans);
+    self->sesman_trans = trans_create(1, 8192, 8192);
+    xrdp_mm_get_sesman_port(port, sizeof(port));
+    g_snprintf(text, 255, "connecting to sesman ip %s port %s", ip, port);
     xrdp_wm_log_msg(self->wm, text);
-    error = g_tcp_connect(self->sck, ip, "3350");
-    if (error == 0)
+
+    self->sesman_trans->trans_data_in = xrdp_mm_sesman_data_in;
+    self->sesman_trans->header_size = 8;
+    self->sesman_trans->callback_data = self;
+    /* try to connect up to 4 times */
+    for (index = 0; index < 4; index++)
     {
-      ok = 1;
-    }
-    else if (error == -1)
-    {
-      if (g_tcp_last_error_would_block(self->sck))
+      if (trans_connect(self->sesman_trans, ip, port, 3000) == 0)
       {
+        self->sesman_trans_up = 1;
         ok = 1;
-        /* wait up to 3 seconds for connect to complete */
-        index = 0;
-        while (!g_tcp_can_send(self->sck, 100))
-        {
-          index++;
-          if ((index >= 30) || g_is_term())
-          {
-            g_snprintf(errstr, 255, "error connect sesman - connect timeout");
-            ok = 0;
-            break;
-          }
-        }
-      }
-      else
-      {
-        name = g_get_strerror();
-        g_snprintf(errstr, 255, "error connect sesman - %s", name);
+        break;
       }
+      g_sleep(1000);
+      g_writeln("xrdp_mm_connect: connect failed "
+                "trying again...");
     }
     if (ok)
     {
       /* fully connect */
       xrdp_wm_log_msg(self->wm, "sesman connect ok");
       self->connected_state = 1;
-      self->wm->pro_layer->app_sck = self->sck;
       rv = xrdp_mm_send_login(self);
     }
     else
     {
       xrdp_wm_log_msg(self->wm, errstr);
-      g_tcp_close(self->sck);
-      self->sck = 0;
+      trans_delete(self->sesman_trans);
+      self->sesman_trans = 0;
+      self->sesman_trans_up = 0;
       rv = 1;
     }
   }
@@ -587,62 +939,96 @@ xrdp_mm_connect(struct xrdp_mm* self)
     {
       if (xrdp_mm_setup_mod2(self) == 0)
       {
-        self->wm->login_mode = 10;
-        self->wm->pro_layer->app_sck = self->mod->sck;
+        xrdp_wm_set_login_mode(self->wm, 10);
       }
     }
     if (self->wm->login_mode != 10)
     {
-      self->wm->pro_layer->app_sck = 0;
-      self->wm->login_mode = 11;
+      xrdp_wm_set_login_mode(self->wm, 11);
       xrdp_mm_module_cleanup(self);
     }
   }
+  self->sesman_controlled = use_sesman;
+  return rv;
+}
+
+/*****************************************************************************/
+int APP_CC
+xrdp_mm_get_wait_objs(struct xrdp_mm* self,
+                      tbus* read_objs, int* rcount,
+                      tbus* write_objs, int* wcount, int* timeout)
+{
+  int rv;
+
+  if (self == 0)
+  {
+    return 0;
+  }
+  rv = 0;
+  if ((self->sesman_trans != 0) && self->sesman_trans_up)
+  {
+    trans_get_wait_objs(self->sesman_trans, read_objs, rcount, timeout);
+  }
+  if ((self->chan_trans != 0) && self->chan_trans_up)
+  {
+    trans_get_wait_objs(self->chan_trans, read_objs, rcount, timeout);
+  }
+  if (self->mod != 0)
+  {
+    if (self->mod->mod_get_wait_objs != 0)
+    {
+      rv = self->mod->mod_get_wait_objs(self->mod, read_objs, rcount,
+                                        write_objs, wcount, timeout);
+    }
+  }
   return rv;
 }
 
 /*****************************************************************************/
 int APP_CC
-xrdp_mm_signal(struct xrdp_mm* self)
+xrdp_mm_check_wait_objs(struct xrdp_mm* self)
 {
-  struct stream* s;
-  int error;
-  int version;
-  int size;
-  int code;
   int rv;
 
-  rv = 0;
-  if (self->connected_state == 1)
+  if (self == 0)
   {
-    make_stream(s);
-    init_stream(s, 8192);
-    error = xrdp_mm_recv(self, s->data, 8);
-    if (error == 0)
+    return 0;
+  }
+  rv = 0;
+  if ((self->sesman_trans != 0) && self->sesman_trans_up)
+  {
+    if (trans_check_wait_objs(self->sesman_trans) != 0)
     {
-      in_uint32_be(s, version);
-      in_uint32_be(s, size);
-      init_stream(s, 8192);
-      error = (size <= 8) || (size > 8000);
+      self->delete_sesman_trans = 1;
     }
-    if (error == 0)
+  }
+  if ((self->chan_trans != 0) && self->chan_trans_up)
+  {
+    if (trans_check_wait_objs(self->chan_trans) != 0)
     {
-      error = xrdp_mm_recv(self, s->data, size - 8);
+      self->delete_chan_trans = 1;
     }
-    if (error == 0)
+  }
+  if (self->mod != 0)
+  {
+    if (self->mod->mod_check_wait_objs != 0)
     {
-      in_uint16_be(s, code);
-      switch (code)
-      {
-        case 3:
-          rv = xrdp_mm_process_login_response(self, s);
-          break;
-        default:
-          g_writeln("unknown code %d in xrdp_mm_signal", code);
-          break;
-      }
+      rv = self->mod->mod_check_wait_objs(self->mod);
     }
-    free_stream(s);
+  }
+  if (self->delete_sesman_trans)
+  {
+    trans_delete(self->sesman_trans);
+    self->sesman_trans = 0;
+    self->sesman_trans_up = 0;
+    self->delete_sesman_trans = 0;
+  }
+  if (self->delete_chan_trans)
+  {
+    trans_delete(self->chan_trans);
+    self->chan_trans = 0;
+    self->chan_trans_up = 0;
+    self->delete_chan_trans = 0;
   }
   return rv;
 }
@@ -654,7 +1040,7 @@ server_begin_update(struct xrdp_mod* mod)
   struct xrdp_wm* wm;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
   p = xrdp_painter_create(wm, wm->session);
   xrdp_painter_begin_update(p);
   mod->painter = (long)p;
@@ -667,7 +1053,7 @@ server_end_update(struct xrdp_mod* mod)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   xrdp_painter_end_update(p);
   xrdp_painter_delete(p);
   mod->painter = 0;
@@ -681,8 +1067,8 @@ server_fill_rect(struct xrdp_mod* mod, int x, int y, int cx, int cy)
   struct xrdp_wm* wm;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  p = (struct xrdp_painter*)mod->painter;
+  wm = (struct xrdp_wm*)(mod->wm);
+  p = (struct xrdp_painter*)(mod->painter);
   xrdp_painter_fill_rect(p, wm->screen, x, y, cx, cy);
   return 0;
 }
@@ -695,8 +1081,8 @@ server_screen_blt(struct xrdp_mod* mod, int x, int y, int cx, int cy,
   struct xrdp_wm* wm;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  p = (struct xrdp_painter*)mod->painter;
+  wm = (struct xrdp_wm*)(mod->wm);
+  p = (struct xrdp_painter*)(mod->painter);
   p->rop = 0xcc;
   xrdp_painter_copy(p, wm->screen, wm->screen, x, y, cx, cy, srcx, srcy);
   return 0;
@@ -711,8 +1097,8 @@ server_paint_rect(struct xrdp_mod* mod, int x, int y, int cx, int cy,
   struct xrdp_bitmap* b;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  p = (struct xrdp_painter*)mod->painter;
+  wm = (struct xrdp_wm*)(mod->wm);
+  p = (struct xrdp_painter*)(mod->painter);
   b = xrdp_bitmap_create_with_data(width, height, wm->screen->bpp, data, wm);
   xrdp_painter_copy(p, b, wm->screen, x, y, cx, cy, srcx, srcy);
   xrdp_bitmap_delete(b);
@@ -726,7 +1112,7 @@ server_set_pointer(struct xrdp_mod* mod, int x, int y,
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
   xrdp_wm_pointer(wm, data, mask, x, y);
   return 0;
 }
@@ -737,7 +1123,7 @@ server_palette(struct xrdp_mod* mod, int* palette)
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
   if (g_memcmp(wm->palette, palette, 255 * sizeof(int)) != 0)
   {
     g_memcpy(wm->palette, palette, 256 * sizeof(int));
@@ -757,7 +1143,7 @@ server_msg(struct xrdp_mod* mod, char* msg, int code)
     g_writeln(msg);
     return 0;
   }
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
   return xrdp_wm_log_msg(wm, msg);
 }
 
@@ -774,7 +1160,7 @@ server_set_clip(struct xrdp_mod* mod, int x, int y, int cx, int cy)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   return xrdp_painter_set_clip(p, x, y, cx, cy);
 }
 
@@ -784,7 +1170,7 @@ server_reset_clip(struct xrdp_mod* mod)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   return xrdp_painter_clr_clip(p);
 }
 
@@ -794,7 +1180,7 @@ server_set_fgcolor(struct xrdp_mod* mod, int fgcolor)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->fg_color = fgcolor;
   p->pen.color = p->fg_color;
   return 0;
@@ -806,7 +1192,7 @@ server_set_bgcolor(struct xrdp_mod* mod, int bgcolor)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->bg_color = bgcolor;
   return 0;
 }
@@ -817,7 +1203,7 @@ server_set_opcode(struct xrdp_mod* mod, int opcode)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->rop = opcode;
   return 0;
 }
@@ -828,7 +1214,7 @@ server_set_mixmode(struct xrdp_mod* mod, int mixmode)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->mix_mode = mixmode;
   return 0;
 }
@@ -840,7 +1226,7 @@ server_set_brush(struct xrdp_mod* mod, int x_orgin, int y_orgin,
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->brush.x_orgin = x_orgin;
   p->brush.y_orgin = y_orgin;
   p->brush.style = style;
@@ -854,7 +1240,7 @@ server_set_pen(struct xrdp_mod* mod, int style, int width)
 {
   struct xrdp_painter* p;
 
-  p = (struct xrdp_painter*)mod->painter;
+  p = (struct xrdp_painter*)(mod->painter);
   p->pen.style = style;
   p->pen.width = width;
   return 0;
@@ -867,8 +1253,8 @@ server_draw_line(struct xrdp_mod* mod, int x1, int y1, int x2, int y2)
   struct xrdp_wm* wm;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  p = (struct xrdp_painter*)mod->painter;
+  wm = (struct xrdp_wm*)(mod->wm);
+  p = (struct xrdp_painter*)(mod->painter);
   return xrdp_painter_line(p, wm->screen, x1, y1, x2, y2);
 }
 
@@ -902,8 +1288,8 @@ server_draw_text(struct xrdp_mod* mod, int font,
   struct xrdp_wm* wm;
   struct xrdp_painter* p;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  p = (struct xrdp_painter*)mod->painter;
+  wm = (struct xrdp_wm*)(mod->wm);
+  p = (struct xrdp_painter*)(mod->painter);
   return xrdp_painter_draw_text2(p, wm->screen, font, flags,
                                  mixmode, clip_left, clip_top,
                                  clip_right, clip_bottom,
@@ -918,7 +1304,7 @@ server_reset(struct xrdp_mod* mod, int width, int height, int bpp)
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
   if (wm->client_info == 0)
   {
     return 1;
@@ -958,28 +1344,43 @@ server_query_channel(struct xrdp_mod* mod, int index, char* channel_name,
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
+  if (wm->mm->sesman_controlled)
+  {
+    return 1;
+  }
   return libxrdp_query_channel(wm->session, index, channel_name,
                                channel_flags);
 }
 
 /*****************************************************************************/
+/* returns -1 on error */
 int DEFAULT_CC
 server_get_channel_id(struct xrdp_mod* mod, char* name)
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
+  wm = (struct xrdp_wm*)(mod->wm);
+  if (wm->mm->sesman_controlled)
+  {
+    return -1;
+  }
   return libxrdp_get_channel_id(wm->session, name);
 }
 
 /*****************************************************************************/
 int DEFAULT_CC
 server_send_to_channel(struct xrdp_mod* mod, int channel_id,
-                       char* data, int data_len)
+                       char* data, int data_len,
+                       int total_data_len, int flags)
 {
   struct xrdp_wm* wm;
 
-  wm = (struct xrdp_wm*)mod->wm;
-  return libxrdp_send_to_channel(wm->session, channel_id, data, data_len);
+  wm = (struct xrdp_wm*)(mod->wm);
+  if (wm->mm->sesman_controlled)
+  {
+    return 1;
+  }
+  return libxrdp_send_to_channel(wm->session, channel_id, data, data_len,
+                                 total_data_len, flags);
 }
