@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2005-2006
+   Copyright (C) Jay Sorg 2005-2009
 
    libxup main header file
 
@@ -26,6 +26,8 @@
 #include "os_calls.h"
 #include "defines.h"
 
+#define CURRENT_MOD_VER 1
+
 struct mod
 {
   int size; /* size of this struct */
@@ -33,12 +35,16 @@ struct mod
   /* client functions */
   int (*mod_start)(struct mod* v, int w, int h, int bpp);
   int (*mod_connect)(struct mod* v);
-  int (*mod_event)(struct mod* v, int msg, long param1, long param2,
-                   long param3, long param4);
+  int (*mod_event)(struct mod* v, int msg, tbus param1, tbus param2,
+                   tbus param3, tbus param4);
   int (*mod_signal)(struct mod* v);
   int (*mod_end)(struct mod* v);
   int (*mod_set_param)(struct mod* v, char* name, char* value);
-  long mod_dumby[100 - 6]; /* align, 100 minus the number of mod
+  int (*mod_session_change)(struct mod* v, int, int);
+  int (*mod_get_wait_objs)(struct mod* v, tbus* read_objs, int* rcount,
+                           tbus* write_objs, int* wcount, int* timeout);
+  int (*mod_check_wait_objs)(struct mod* v);
+  tbus mod_dumby[100 - 9]; /* align, 100 minus the number of mod
                               functions above */
   /* server functions */
   int (*server_begin_update)(struct mod* v);
@@ -78,13 +84,14 @@ struct mod
                               int* channel_flags);
   int (*server_get_channel_id)(struct mod* v, char* name);
   int (*server_send_to_channel)(struct mod* v, int channel_id,
-                                char* data, int data_len);
-  long server_dumby[100 - 24]; /* align, 100 minus the number of server
+                                char* data, int data_len,
+                                int total_data_len, int flags);
+  tbus server_dumby[100 - 24]; /* align, 100 minus the number of server
                                   functions above */
   /* common */
-  long handle; /* pointer to self as long */
-  long wm;
-  long painter;
+  tbus handle; /* pointer to self as long */
+  tbus wm;
+  tbus painter;
   int sck;
   /* mod data */
   int width;
@@ -95,4 +102,5 @@ struct mod
   char password[256];
   char ip[256];
   char port[256];
+  tbus sck_obj;
 };
